@@ -1,4 +1,7 @@
+# flake8: noqa
 from nbmanips.notebook import Notebook
+from nbmanips.notebook import IPYNB, DBC, ZPLN
+import nbmanips.exporters
 
 from os.path import dirname as _dir
 from os.path import join as _join
