@@ -12,6 +12,9 @@ except ImportError:
     pygments = None
     get_lexer_by_name = None
 
+from nbconvert.filters.markdown_mistune import IPythonRenderer, MarkdownWithMath
+from bs4 import BeautifulSoup
+
 from nbmanips.cell_utils import printable_cell, FORMATTER, monochrome
 from nbmanips.cell_output import CellOutput
 from nbmanips.utils import total_size
@@ -258,6 +261,8 @@ class CodeCell(Cell, cell_type="code"):
 
 
 class MarkdownCell(Cell, cell_type="markdown"):
+    _bs4_parser = 'lxml'
+
     def to_str(self, width=None, style='single', use_pygments=None, pygments_lexer=None, color=None, **kwargs):
         use_pygments = pygments is not None if use_pygments is None else use_pygments
 
@@ -268,6 +273,23 @@ class MarkdownCell(Cell, cell_type="markdown"):
         else:
             return self.source
 
+    @property
+    def attachments(self):
+        return self.cell.get('attachments', {})
+
+    @property
+    def html(self):
+        renderer = IPythonRenderer(
+            escape=False,
+            attachments=self.attachments,
+            exclude_anchor_links=True
+        )
+        return MarkdownWithMath(renderer=renderer).render(self.source)
+
+    @property
+    def soup(self):
+        return BeautifulSoup(self.html, self._bs4_parser)
+
 
 class RawCell(Cell, cell_type="raw"):
     pass
